/***************************************************************************************************************************************
*	File name 	: 	SPI_MMode.C
*	Purpose		: 	This file has functions to initialize SPI controller in Memory mapped mode. It also configures cache controller.
****************************************************************************************************************************************/
#include "main.h"

void SetupCache();
void ReadCacheConfig(unsigned int* CacheCFGAddr);
void WriteCacheConfig(unsigned int* CacheCFGAddr);


/****************************************************************************************************************************************
*	Function name:  SetupSPI_MMode
*	Description	:	The FFT and filter co-efficients (used by SNR_calculation function) are stored in internal SPI flash memory. 
*					This SPI memory can be memory mapped for read accesses using the "Memory map" mode of SPI controller. 
*					This memory mapped mode is configured for following setting:
						- Memory Map Mode enabled
					  - SPI CLock Freq = SCLK freq
					  - SPI Quad IO mode - opcode: 0xEB 
					  - LAG = LEAD = 1, STOP = 3
					  - Fast mode enabled
					  - Merge enabled 
					  - Wrap disabled
					  - No of Addr pins = 4
					  - No of Cmd pins = 1
					  - Dummy = 5 bytes  
					  - XIP disabled
*					This function internally calls SetupCache () just before enabling SPI controller,
*	Arguments	:	None. 
*   Return value:   None.
*   Note		: 	This function is called at the start only by Init_Display function.
****************************************************************************************************************************************/

void SetupSPI_MMode(void)
{
	//first disable SPI
	*pREG_SPI2_CTL = 0;
	*pREG_SPI2_MMRDH = 0;
	__DSB();
  
	*pREG_SPI2_CLK = 	0x00;
	*pREG_SPI2_DLY = (	((3 << BITP_SPI_DLY_STOP ) & BITM_SPI_DLY_STOP ) |
						((1 << BITP_SPI_DLY_LEADX) & BITM_SPI_DLY_LEADX) |
						((1  << BITP_SPI_DLY_LAGX) & BITM_SPI_DLY_LAGX ) );
	  
	*pREG_SPI2_CTL = ( 	ENUM_SPI_CTL_MM_EN | ENUM_SPI_CTL_MASTER | ENUM_SPI_CTL_SIZE32 | 
						ENUM_SPI_CTL_HW_SSEL | ENUM_SPI_CTL_ASSRT_SSEL | ENUM_SPI_CTL_MSB_FIRST | 
						ENUM_SPI_CTL_FAST_EN | ENUM_SPI_CTL_MIO_QUAD   |
						((0 << BITP_SPI_CTL_CPHA) & BITM_SPI_CTL_CPHA) |
						((0 << BITP_SPI_CTL_CPOL) & BITM_SPI_CTL_CPOL)	);
	
	*pREG_SPI2_TXCTL =	(BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TEN );  
	*pREG_SPI2_RXCTL =	(BITM_SPI_RXCTL_REN );
	
	*pREG_SPI2_MMRDH = (((0xEB << BITP_SPI_MMRDH_OPCODE)& BITM_SPI_MMRDH_OPCODE	)	|
						((3 << BITP_SPI_MMRDH_ADRSIZE) 	& BITM_SPI_MMRDH_ADRSIZE)	|
						((1 << BITP_SPI_MMRDH_ADRPINS) 	& BITM_SPI_MMRDH_ADRPINS) 	|
						((5	<< BITP_SPI_MMRDH_DMYSIZE) 	& BITM_SPI_MMRDH_DMYSIZE) 	|
						((0xFF << BITP_SPI_MMRDH_MODE)	& BITM_SPI_MMRDH_MODE	)	|
						((1 << BITP_SPI_MMRDH_TRIDMY)	& BITM_SPI_MMRDH_TRIDMY	)	|
						((1 << BITP_SPI_MMRDH_MERGE	)	& BITM_SPI_MMRDH_MERGE	)	|
						((0 << BITP_SPI_MMRDH_WRAP	)	& BITM_SPI_MMRDH_WRAP	)	|
						((0 << BITP_SPI_MMRDH_CMDSKIP)	& BITM_SPI_MMRDH_CMDSKIP)	|	 
						((0 << BITP_SPI_MMRDH_CMDPINS)	& BITM_SPI_MMRDH_CMDPINS)	);
	
	*pREG_SPI2_MMTOP = 0x18000000 + (SPI_FLASH_BLOCK_SIZE * SPI_FLASH_BLOCK_COUNT);

	*pREG_SPI2_SLVSEL = ENUM_SPI_SLVSEL_SSEL1_HI | ENUM_SPI_SLVSEL_SSEL1_EN;   
	__DSB();  __ISB();

	SetupCache();

  //Now enable SPI
  *pREG_SPI2_CTL |= BITM_SPI_CTL_EN;
  __DSB();
}


/****************************************************************************************************************************************
*	Function name:  SetupCache
*	Description	:	This function configures the cache controller to followinf settings:
*					cache enable, 32-byte data cache line, line based first (WRAP disable), Enable speculative fetch. 
*					It also intialises Memory translators MEM X and MEM Y to internal SPI memory and external SMC memory respectively.
*	Arguments	:	None. 
*   Return value:   None.
*   Note		: 	This function is internally called by SetupSPI_MMode() fucntion just before enabling SPI in memory mapped mode.
****************************************************************************************************************************************/

//structure for bits of config;
struct CacheCFGREG
{
  unsigned CMODE:2;
  unsigned PEMODE:2; 
  unsigned CLEAR:1;
  unsigned CBEN:1; 
  unsigned NOPRE:1; 
  unsigned CORG:3;
  unsigned ILINE:2;
  unsigned DLINE:2;
  unsigned ILBF:1;
  unsigned DLBF:1;
  unsigned PLIM:3;
  unsigned PMSK:1;
  unsigned PCNT:3;
  unsigned PERR:1;
  unsigned PIRQ:1;
  unsigned NOSPEC:1;
  unsigned Reserved:4;
  unsigned TRAN0:1;
  unsigned TRAN1:1; 
};

struct CacheCFGREG CCFG;

void SetupCache(void)
{
    *pREG_M4P0_CACHE_MEMX = 0x18000000 | 0x3;
	*pREG_M4P0_CACHE_MEMY = 0x60000000 | 0x3;
	__DSB();
  
	ReadCacheConfig((unsigned int*)&CCFG);
	__DSB();
	CCFG.CMODE    = 0;			// set cache enable  3:disable
	CCFG.PEMODE   = 0;			// set cache enable, 3: disable
	CCFG.CLEAR    = 0;
	CCFG.CBEN     = 0;			// disable backdoor feature
	CCFG.NOPRE    = 1;			// disable preemptive feature
	CCFG.CORG     = 0;			// shared 4 ways
	CCFG.PMSK     = 0;			// Parity Mask
	CCFG.TRAN0    = 0;			// Translate0
	CCFG.TRAN1    = 0;			// Translate1
	CCFG.NOSPEC   = 0;			// Enable Speculative feature  
	CCFG.ILBF     = 1;			//Line base First 
	CCFG.DLBF     = 1; 
	CCFG.ILINE    = 3;
  	CCFG.DLINE    = 3;

	WriteCacheConfig((unsigned int*)&CCFG);  
	ReadCacheConfig((unsigned int*)&CCFG); 
	
//	ClearCache();
	*pREG_M4P0_CACHE_CFG |= BITM_M4P_CACHE_CFG_CLEAR;
	__DSB();
	while(*pREG_M4P0_CACHE_CFG & BITM_M4P_CACHE_CFG_CLEAR)
	__NOP(); __DSB();
}
//----------------------------------------------------------

void WriteCacheConfig(unsigned int* CacheCFGAddr)
{
     *pREG_M4P0_CACHE_CFG  = *CacheCFGAddr;
  __DSB();
}
//----------------------------------------------------------

void ReadCacheConfig(unsigned int* CacheCFGAddr)
{ 
  *CacheCFGAddr = *pREG_M4P0_CACHE_CFG;
   __DSB();
}
/****************************************************************************************************************************************/
